import sinon from 'sinon'
import dmpTrack, {onLoad} from '../../src/analytics/dmp/track'
const axios = require('axios')

const VISITOR_ID = 'fakeVisitorId'

describe('dmp track', () => {
  let axiosStub
  let setItemStub

  beforeEach(() => {
    axiosStub = sinon.stub(axios, 'get').resolves({
      stuff: [
        {
          cv: '1234*5678'
        }
      ]
    })

    setItemStub = sinon.spy()
  })

  afterEach(() => {
    axiosStub.restore()
  })

  it('send track event to the dmp', async () => {
    await dmpTrack({
      visitorId: VISITOR_ID,
      imageFactory: () => {
        return {}
      },
      storageFactory: () => ({
        setItem: setItemStub
      })
    })
    sinon.assert.callCount(axiosStub, 1)
    sinon.assert.calledWith(axiosStub, 'https://mpispain.demdex.net/event', {
      params: {
        d_cts: 2,
        d_full: 1,
        d_mid: 'fakeVisitorId',
        d_orgid: '05FF6243578784B37F000101@AdobeOrg',
        d_rtbd: 'json'
      }
    })
    sinon.assert.callCount(setItemStub, 2)
    sinon.assert.match(setItemStub.getCall(0).args[0], 'dmp:track')
    sinon.assert.match(setItemStub.getCall(0).args[1], 'fakeVisitorId')
    sinon.assert.match(setItemStub.getCall(0).args[0], 'dmp:track')
    sinon.assert.match(setItemStub.getCall(0).args[1], '')
  })

  it('loads the google and facebook pixels', async () => {
    const images = []
    await dmpTrack({
      visitorId: VISITOR_ID,
      imageFactory: () => {
        const image = {}
        images.push(image)
        return image
      },
      storageFactory: () => ({setItem: setItemStub})
    })
    sinon.assert.match(images.length, 2)
    sinon.assert.match(
      images[0].src,
      'https://googleads.g.doubleclick.net/pagead/viewthroughconversion/1024323628/?guid=ON&script=0&data=aam=1234;aam=5678&value=0'
    )
    sinon.assert.match(
      images[1].src,
      'https://www.facebook.com/tr/?id=283351338713870&ev=Adobe-Audience-Manager-Segment&cd[segId]=1234,5678&noscript=1'
    )
    sinon.assert.callCount(setItemStub, 2)
    sinon.assert.match(setItemStub.getCall(0).args[0], 'dmp:track')
    sinon.assert.match(setItemStub.getCall(0).args[1], 'fakeVisitorId')
    sinon.assert.match(setItemStub.getCall(1).args[0], 'dmp:track')
    sinon.assert.match(setItemStub.getCall(1).args[1], '')
  })
})

describe('when the prior sync dit not resolve', () => {
  let setItemStub
  let getItemStub
  let axiosStub

  beforeEach(async () => {
    getItemStub = sinon.stub().returns('retry')
    setItemStub = sinon.spy()
    axiosStub = sinon.stub(axios, 'get').resolves({
      stuff: [
        {
          cv: '1234*5678'
        }
      ]
    })
  })

  afterEach(() => {
    getItemStub.reset()
    axiosStub.reset()
  })

  it('should retry on load', async () => {
    await onLoad({
      imageFactory: () => ({}),
      storageFactory: () => ({
        getItem: getItemStub,
        setItem: setItemStub
      })
    })

    sinon.assert.callCount(getItemStub, 1)
    sinon.assert.match(getItemStub.getCall(0).args[0], 'dmp:track')
    sinon.assert.match(setItemStub.getCall(0).args[0], 'dmp:track')
    sinon.assert.match(setItemStub.getCall(0).args[1], 'retry')
    sinon.assert.match(setItemStub.getCall(1).args[0], 'dmp:track')
    sinon.assert.match(setItemStub.getCall(1).args[1], '')
  })
})
